# flake8: noqa
# generated by 'xml2py',
# flags '-kdefst -d -c -o _ftd2xx_linux.py ftd2xx_linux.xml -l libftd2xx.so'
# from ctypes import CDLL, c_char_p, c_char,
from ctypes import (
    CFUNCTYPE,
    POINTER,
    Structure,
    CDLL,
    c_char,
    c_char_p,
    c_int,
    c_uint,
    c_short,
    c_ubyte,
    c_long,
    c_ulong,
    c_longlong,
    c_ulonglong,
    c_ushort,
    c_void_p,
)
from typing import List

_libraries = {}
_libraries["libftd2xx.so"] = CDLL("libftd2xx.so")
STRING = c_char_p

FT_DEVICE_BM = 0
FT_DEVICE_AM = 1
FT_DEVICE_100AX = 2
FT_DEVICE_UNKNOWN = 3
FT_DEVICE_2232C = 4
FT_DEVICE_232R = 5

FT_OK = 0
FT_INVALID_HANDLE = 1
FT_DEVICE_NOT_FOUND = 2
FT_DEVICE_NOT_OPENED = 3
FT_IO_ERROR = 4
FT_INSUFFICIENT_RESOURCES = 5
FT_INVALID_PARAMETER = 6
FT_INVALID_BAUD_RATE = 7
FT_DEVICE_NOT_OPENED_FOR_ERASE = 8
FT_DEVICE_NOT_OPENED_FOR_WRITE = 9
FT_FAILED_TO_WRITE_DEVICE = 10
FT_EEPROM_READ_FAILED = 11
FT_EEPROM_WRITE_FAILED = 12
FT_EEPROM_ERASE_FAILED = 13
FT_EEPROM_NOT_PRESENT = 14
FT_EEPROM_NOT_PROGRAMMED = 15
FT_INVALID_ARGS = 16
FT_NOT_SUPPORTED = 17
FT_OTHER_ERROR = 18


# /usr/include/bits/pthreadtypes.h 27


class _pthread_fastlock(Structure):
    _fields_ = [
        # /usr/include/bits/pthreadtypes.h 27
        ("__status", c_long),
        ("__spinlock", c_int),
    ]


# /usr/include/bits/pthreadtypes.h 35


class _pthread_descr_struct(Structure):
    _fields_: List[tuple] = [
        # /usr/include/bits/pthreadtypes.h 35
    ]


_pthread_descr = POINTER(_pthread_descr_struct)
# /usr/include/bits/pthreadtypes.h 42


# /usr/include/bits/pthreadtypes.h 70


class pthread_cond_t(Structure):
    _fields_ = [
        # /usr/include/bits/pthreadtypes.h 70
        ("__c_lock", _pthread_fastlock),
        ("__c_waiting", _pthread_descr),
        ("__padding", c_char * 16),
        ("__align", c_longlong),
    ]


pthread_key_t = c_uint
# /usr/include/bits/pthreadtypes.h 93


class pthread_mutex_t(Structure):
    _fields_ = [
        # /usr/include/bits/pthreadtypes.h 93
        ("__m_reserved", c_int),
        ("__m_count", c_int),
        ("__m_owner", _pthread_descr),
        ("__m_kind", c_int),
        ("__m_lock", _pthread_fastlock),
    ]


DWORD = c_ulong
ULONG = c_ulong
USHORT = c_ushort
SHORT = c_short
UCHAR = c_ubyte
WORD = c_ushort
BYTE = c_ubyte
LPBYTE = POINTER(c_ubyte)
BOOL = c_int
BOOLEAN = c_char
CHAR = c_char
LPBOOL = POINTER(c_int)
PUCHAR = POINTER(c_ubyte)
LPCSTR = STRING
PCHAR = STRING
PVOID = c_void_p
HANDLE = c_void_p
LONG = c_long
INT = c_int
UINT = c_uint
LPSTR = STRING
LPTSTR = STRING
LPDWORD = POINTER(DWORD)
LPWORD = POINTER(WORD)
PULONG = POINTER(ULONG)
LPVOID = PVOID
VOID: None = None
ULONGLONG = c_ulonglong
# WinTypes.h 38


class _OVERLAPPED(Structure):
    _fields_ = [
        # WinTypes.h 38
        ("Internal", DWORD),
        ("InternalHigh", DWORD),
        ("Offset", DWORD),
        ("OffsetHigh", DWORD),
        ("hEvent", HANDLE),
    ]


LPOVERLAPPED = POINTER(_OVERLAPPED)
OVERLAPPED = _OVERLAPPED
# WinTypes.h 46


class _SECURITY_ATTRIBUTES(Structure):
    _fields_ = [
        # WinTypes.h 46
        ("nLength", DWORD),
        ("lpSecurityDescriptor", LPVOID),
        ("bInheritHandle", BOOL),
    ]


LPSECURITY_ATTRIBUTES = POINTER(_SECURITY_ATTRIBUTES)
SECURITY_ATTRIBUTES = _SECURITY_ATTRIBUTES
# WinTypes.h 52


class _EVENT_HANDLE(Structure):
    _fields_ = [
        # ftd2xx_linux.h 63
        ("eCondVar", pthread_cond_t),
        ("eMutex", pthread_mutex_t),
        ("iVar", c_int),
    ]


EVENT_HANDLE = _EVENT_HANDLE
FT_HANDLE = POINTER(DWORD)
FT_STATUS = ULONG

# values for unnamed enumeration
PFT_EVENT_HANDLER = CFUNCTYPE(None, c_ulong, c_ulong)
FT_DEVICE = ULONG

# values for unnamed enumeration
# ftd2xx_linux.h 220
FT_Open = _libraries["libftd2xx.so"].FT_Open
FT_Open.restype = FT_STATUS
# FT_Open(deviceNumber, pHandle)
FT_Open.argtypes = [c_int, POINTER(FT_HANDLE)]
FT_Open.__doc__ = """FT_STATUS FT_Open(int deviceNumber, FT_HANDLE * pHandle)
ftd2xx_linux.h:220"""
# ftd2xx_linux.h 227
FT_OpenEx = _libraries["libftd2xx.so"].FT_OpenEx
FT_OpenEx.restype = FT_STATUS
# FT_OpenEx(pArg1, Flags, pHandle)
FT_OpenEx.argtypes = [PVOID, DWORD, POINTER(FT_HANDLE)]
FT_OpenEx.__doc__ = """FT_STATUS FT_OpenEx(PVOID pArg1, DWORD Flags, FT_HANDLE * pHandle)
ftd2xx_linux.h:227"""
# ftd2xx_linux.h 234
FT_ListDevices = _libraries["libftd2xx.so"].FT_ListDevices
FT_ListDevices.restype = FT_STATUS
# FT_ListDevices(pArg1, pArg2, Flags)
FT_ListDevices.argtypes = [PVOID, PVOID, DWORD]
FT_ListDevices.__doc__ = """FT_STATUS FT_ListDevices(PVOID pArg1, PVOID pArg2, DWORD Flags)
ftd2xx_linux.h:234"""
# ftd2xx_linux.h 240
FT_SetVIDPID = _libraries["libftd2xx.so"].FT_SetVIDPID
FT_SetVIDPID.restype = FT_STATUS
# FT_SetVIDPID(dwVID, dwPID)
FT_SetVIDPID.argtypes = [DWORD, DWORD]
FT_SetVIDPID.__doc__ = """FT_STATUS FT_SetVIDPID(DWORD dwVID, DWORD dwPID)
ftd2xx_linux.h:240"""
# ftd2xx_linux.h 246
FT_GetVIDPID = _libraries["libftd2xx.so"].FT_GetVIDPID
FT_GetVIDPID.restype = FT_STATUS
# FT_GetVIDPID(pdwVID, pdwPID)
FT_GetVIDPID.argtypes = [POINTER(DWORD), POINTER(DWORD)]
FT_GetVIDPID.__doc__ = """FT_STATUS FT_GetVIDPID(DWORD * pdwVID, DWORD * pdwPID)
ftd2xx_linux.h:246"""
# ftd2xx_linux.h 251
FT_Close = _libraries["libftd2xx.so"].FT_Close
FT_Close.restype = FT_STATUS
# FT_Close(ftHandle)
FT_Close.argtypes = [FT_HANDLE]
FT_Close.__doc__ = """FT_STATUS FT_Close(FT_HANDLE ftHandle)
ftd2xx_linux.h:251"""
# ftd2xx_linux.h 259
FT_Read = _libraries["libftd2xx.so"].FT_Read
FT_Read.restype = FT_STATUS
# FT_Read(ftHandle, lpBuffer, nBufferSize, lpBytesReturned)
FT_Read.argtypes = [FT_HANDLE, LPVOID, DWORD, LPDWORD]
FT_Read.__doc__ = """FT_STATUS FT_Read(FT_HANDLE ftHandle, LPVOID lpBuffer, DWORD nBufferSize, LPDWORD lpBytesReturned)
ftd2xx_linux.h:259"""
# ftd2xx_linux.h 267
FT_Write = _libraries["libftd2xx.so"].FT_Write
FT_Write.restype = FT_STATUS
# FT_Write(ftHandle, lpBuffer, nBufferSize, lpBytesWritten)
FT_Write.argtypes = [FT_HANDLE, LPVOID, DWORD, LPDWORD]
FT_Write.__doc__ = """FT_STATUS FT_Write(FT_HANDLE ftHandle, LPVOID lpBuffer, DWORD nBufferSize, LPDWORD lpBytesWritten)
ftd2xx_linux.h:267"""
# ftd2xx_linux.h 279
FT_IoCtl = _libraries["libftd2xx.so"].FT_IoCtl
FT_IoCtl.restype = FT_STATUS
# FT_IoCtl(ftHandle, dwIoControlCode, lpInBuf, nInBufSize, lpOutBuf, nOutBufSize, lpBytesReturned, lpOverlapped)
FT_IoCtl.argtypes = [
    FT_HANDLE,
    DWORD,
    LPVOID,
    DWORD,
    LPVOID,
    DWORD,
    LPDWORD,
    LPOVERLAPPED,
]
FT_IoCtl.__doc__ = """FT_STATUS FT_IoCtl(FT_HANDLE ftHandle, DWORD dwIoControlCode, LPVOID lpInBuf, DWORD nInBufSize, LPVOID lpOutBuf, DWORD nOutBufSize, LPDWORD lpBytesReturned, LPOVERLAPPED lpOverlapped)
ftd2xx_linux.h:279"""
# ftd2xx_linux.h 285
FT_SetBaudRate = _libraries["libftd2xx.so"].FT_SetBaudRate
FT_SetBaudRate.restype = FT_STATUS
# FT_SetBaudRate(ftHandle, BaudRate)
FT_SetBaudRate.argtypes = [FT_HANDLE, ULONG]
FT_SetBaudRate.__doc__ = """FT_STATUS FT_SetBaudRate(FT_HANDLE ftHandle, ULONG BaudRate)
ftd2xx_linux.h:285"""
# ftd2xx_linux.h 291
FT_SetDivisor = _libraries["libftd2xx.so"].FT_SetDivisor
FT_SetDivisor.restype = FT_STATUS
# FT_SetDivisor(ftHandle, Divisor)
FT_SetDivisor.argtypes = [FT_HANDLE, USHORT]
FT_SetDivisor.__doc__ = """FT_STATUS FT_SetDivisor(FT_HANDLE ftHandle, USHORT Divisor)
ftd2xx_linux.h:291"""
# ftd2xx_linux.h 299
FT_SetDataCharacteristics = _libraries["libftd2xx.so"].FT_SetDataCharacteristics
FT_SetDataCharacteristics.restype = FT_STATUS
# FT_SetDataCharacteristics(ftHandle, WordLength, StopBits, Parity)
FT_SetDataCharacteristics.argtypes = [FT_HANDLE, UCHAR, UCHAR, UCHAR]
FT_SetDataCharacteristics.__doc__ = """FT_STATUS FT_SetDataCharacteristics(FT_HANDLE ftHandle, UCHAR WordLength, UCHAR StopBits, UCHAR Parity)
ftd2xx_linux.h:299"""
# ftd2xx_linux.h 307
FT_SetFlowControl = _libraries["libftd2xx.so"].FT_SetFlowControl
FT_SetFlowControl.restype = FT_STATUS
# FT_SetFlowControl(ftHandle, FlowControl, XonChar, XoffChar)
FT_SetFlowControl.argtypes = [FT_HANDLE, USHORT, UCHAR, UCHAR]
FT_SetFlowControl.__doc__ = """FT_STATUS FT_SetFlowControl(FT_HANDLE ftHandle, USHORT FlowControl, UCHAR XonChar, UCHAR XoffChar)
ftd2xx_linux.h:307"""
# ftd2xx_linux.h 312
FT_ResetDevice = _libraries["libftd2xx.so"].FT_ResetDevice
FT_ResetDevice.restype = FT_STATUS
# FT_ResetDevice(ftHandle)
FT_ResetDevice.argtypes = [FT_HANDLE]
FT_ResetDevice.__doc__ = """FT_STATUS FT_ResetDevice(FT_HANDLE ftHandle)
ftd2xx_linux.h:312"""
# ftd2xx_linux.h 317
FT_SetDtr = _libraries["libftd2xx.so"].FT_SetDtr
FT_SetDtr.restype = FT_STATUS
# FT_SetDtr(ftHandle)
FT_SetDtr.argtypes = [FT_HANDLE]
FT_SetDtr.__doc__ = """FT_STATUS FT_SetDtr(FT_HANDLE ftHandle)
ftd2xx_linux.h:317"""
# ftd2xx_linux.h 322
FT_ClrDtr = _libraries["libftd2xx.so"].FT_ClrDtr
FT_ClrDtr.restype = FT_STATUS
# FT_ClrDtr(ftHandle)
FT_ClrDtr.argtypes = [FT_HANDLE]
FT_ClrDtr.__doc__ = """FT_STATUS FT_ClrDtr(FT_HANDLE ftHandle)
ftd2xx_linux.h:322"""
# ftd2xx_linux.h 327
FT_SetRts = _libraries["libftd2xx.so"].FT_SetRts
FT_SetRts.restype = FT_STATUS
# FT_SetRts(ftHandle)
FT_SetRts.argtypes = [FT_HANDLE]
FT_SetRts.__doc__ = """FT_STATUS FT_SetRts(FT_HANDLE ftHandle)
ftd2xx_linux.h:327"""
# ftd2xx_linux.h 332
FT_ClrRts = _libraries["libftd2xx.so"].FT_ClrRts
FT_ClrRts.restype = FT_STATUS
# FT_ClrRts(ftHandle)
FT_ClrRts.argtypes = [FT_HANDLE]
FT_ClrRts.__doc__ = """FT_STATUS FT_ClrRts(FT_HANDLE ftHandle)
ftd2xx_linux.h:332"""
# ftd2xx_linux.h 338
FT_GetModemStatus = _libraries["libftd2xx.so"].FT_GetModemStatus
FT_GetModemStatus.restype = FT_STATUS
# FT_GetModemStatus(ftHandle, pModemStatus)
FT_GetModemStatus.argtypes = [FT_HANDLE, POINTER(ULONG)]
FT_GetModemStatus.__doc__ = """FT_STATUS FT_GetModemStatus(FT_HANDLE ftHandle, ULONG * pModemStatus)
ftd2xx_linux.h:338"""
# ftd2xx_linux.h 347
FT_SetChars = _libraries["libftd2xx.so"].FT_SetChars
FT_SetChars.restype = FT_STATUS
# FT_SetChars(ftHandle, EventChar, EventCharEnabled, ErrorChar, ErrorCharEnabled)
FT_SetChars.argtypes = [FT_HANDLE, UCHAR, UCHAR, UCHAR, UCHAR]
FT_SetChars.__doc__ = """FT_STATUS FT_SetChars(FT_HANDLE ftHandle, UCHAR EventChar, UCHAR EventCharEnabled, UCHAR ErrorChar, UCHAR ErrorCharEnabled)
ftd2xx_linux.h:347"""
# ftd2xx_linux.h 353
FT_Purge = _libraries["libftd2xx.so"].FT_Purge
FT_Purge.restype = FT_STATUS
# FT_Purge(ftHandle, Mask)
FT_Purge.argtypes = [FT_HANDLE, ULONG]
FT_Purge.__doc__ = """FT_STATUS FT_Purge(FT_HANDLE ftHandle, ULONG Mask)
ftd2xx_linux.h:353"""
# ftd2xx_linux.h 360
FT_SetTimeouts = _libraries["libftd2xx.so"].FT_SetTimeouts
FT_SetTimeouts.restype = FT_STATUS
# FT_SetTimeouts(ftHandle, ReadTimeout, WriteTimeout)
FT_SetTimeouts.argtypes = [FT_HANDLE, ULONG, ULONG]
FT_SetTimeouts.__doc__ = """FT_STATUS FT_SetTimeouts(FT_HANDLE ftHandle, ULONG ReadTimeout, ULONG WriteTimeout)
ftd2xx_linux.h:360"""
# ftd2xx_linux.h 366
FT_GetQueueStatus = _libraries["libftd2xx.so"].FT_GetQueueStatus
FT_GetQueueStatus.restype = FT_STATUS
# FT_GetQueueStatus(ftHandle, dwRxBytes)
FT_GetQueueStatus.argtypes = [FT_HANDLE, POINTER(DWORD)]
FT_GetQueueStatus.__doc__ = """FT_STATUS FT_GetQueueStatus(FT_HANDLE ftHandle, DWORD * dwRxBytes)
ftd2xx_linux.h:366"""
# ftd2xx_linux.h 373
FT_SetEventNotification = _libraries["libftd2xx.so"].FT_SetEventNotification
FT_SetEventNotification.restype = FT_STATUS
# FT_SetEventNotification(ftHandle, Mask, Param)
FT_SetEventNotification.argtypes = [FT_HANDLE, DWORD, PVOID]
FT_SetEventNotification.__doc__ = """FT_STATUS FT_SetEventNotification(FT_HANDLE ftHandle, DWORD Mask, PVOID Param)
ftd2xx_linux.h:373"""
# ftd2xx_linux.h 381
FT_GetStatus = _libraries["libftd2xx.so"].FT_GetStatus
FT_GetStatus.restype = FT_STATUS
# FT_GetStatus(ftHandle, dwRxBytes, dwTxBytes, dwEventDWord)
FT_GetStatus.argtypes = [FT_HANDLE, POINTER(DWORD), POINTER(DWORD), POINTER(DWORD)]
FT_GetStatus.__doc__ = """FT_STATUS FT_GetStatus(FT_HANDLE ftHandle, DWORD * dwRxBytes, DWORD * dwTxBytes, DWORD * dwEventDWord)
ftd2xx_linux.h:381"""
# ftd2xx_linux.h 386
FT_SetBreakOn = _libraries["libftd2xx.so"].FT_SetBreakOn
FT_SetBreakOn.restype = FT_STATUS
# FT_SetBreakOn(ftHandle)
FT_SetBreakOn.argtypes = [FT_HANDLE]
FT_SetBreakOn.__doc__ = """FT_STATUS FT_SetBreakOn(FT_HANDLE ftHandle)
ftd2xx_linux.h:386"""
# ftd2xx_linux.h 391
FT_SetBreakOff = _libraries["libftd2xx.so"].FT_SetBreakOff
FT_SetBreakOff.restype = FT_STATUS
# FT_SetBreakOff(ftHandle)
FT_SetBreakOff.argtypes = [FT_HANDLE]
FT_SetBreakOff.__doc__ = """FT_STATUS FT_SetBreakOff(FT_HANDLE ftHandle)
ftd2xx_linux.h:391"""
# ftd2xx_linux.h 397
FT_SetWaitMask = _libraries["libftd2xx.so"].FT_SetWaitMask
FT_SetWaitMask.restype = FT_STATUS
# FT_SetWaitMask(ftHandle, Mask)
FT_SetWaitMask.argtypes = [FT_HANDLE, DWORD]
FT_SetWaitMask.__doc__ = """FT_STATUS FT_SetWaitMask(FT_HANDLE ftHandle, DWORD Mask)
ftd2xx_linux.h:397"""
# ftd2xx_linux.h 403
FT_WaitOnMask = _libraries["libftd2xx.so"].FT_WaitOnMask
FT_WaitOnMask.restype = FT_STATUS
# FT_WaitOnMask(ftHandle, Mask)
FT_WaitOnMask.argtypes = [FT_HANDLE, POINTER(DWORD)]
FT_WaitOnMask.__doc__ = """FT_STATUS FT_WaitOnMask(FT_HANDLE ftHandle, DWORD * Mask)
ftd2xx_linux.h:403"""
# ftd2xx_linux.h 409
FT_GetEventStatus = _libraries["libftd2xx.so"].FT_GetEventStatus
FT_GetEventStatus.restype = FT_STATUS
# FT_GetEventStatus(ftHandle, dwEventDWord)
FT_GetEventStatus.argtypes = [FT_HANDLE, POINTER(DWORD)]
FT_GetEventStatus.__doc__ = """FT_STATUS FT_GetEventStatus(FT_HANDLE ftHandle, DWORD * dwEventDWord)
ftd2xx_linux.h:409"""
# ftd2xx_linux.h 416
FT_ReadEE = _libraries["libftd2xx.so"].FT_ReadEE
FT_ReadEE.restype = FT_STATUS
# FT_ReadEE(ftHandle, dwWordOffset, lpwValue)
FT_ReadEE.argtypes = [FT_HANDLE, DWORD, LPWORD]
FT_ReadEE.__doc__ = """FT_STATUS FT_ReadEE(FT_HANDLE ftHandle, DWORD dwWordOffset, LPWORD lpwValue)
ftd2xx_linux.h:416"""
# ftd2xx_linux.h 423
FT_WriteEE = _libraries["libftd2xx.so"].FT_WriteEE
FT_WriteEE.restype = FT_STATUS
# FT_WriteEE(ftHandle, dwWordOffset, wValue)
FT_WriteEE.argtypes = [FT_HANDLE, DWORD, WORD]
FT_WriteEE.__doc__ = """FT_STATUS FT_WriteEE(FT_HANDLE ftHandle, DWORD dwWordOffset, WORD wValue)
ftd2xx_linux.h:423"""
# ftd2xx_linux.h 428
FT_EraseEE = _libraries["libftd2xx.so"].FT_EraseEE
FT_EraseEE.restype = FT_STATUS
# FT_EraseEE(ftHandle)
FT_EraseEE.argtypes = [FT_HANDLE]
FT_EraseEE.__doc__ = """FT_STATUS FT_EraseEE(FT_HANDLE ftHandle)
ftd2xx_linux.h:428"""
# ftd2xx_linux.h 433


class ft_program_data(Structure):
    _fields_ = [
        # ftd2xx_linux.h 433
        ("Signature1", DWORD),
        ("Signature2", DWORD),
        ("Version", DWORD),
        ("VendorId", WORD),
        ("ProductId", WORD),
        ("Manufacturer", STRING),
        ("ManufacturerId", STRING),
        ("Description", STRING),
        ("SerialNumber", STRING),
        ("MaxPower", WORD),
        ("PnP", WORD),
        ("SelfPowered", WORD),
        ("RemoteWakeup", WORD),
        ("Rev4", UCHAR),
        ("IsoIn", UCHAR),
        ("IsoOut", UCHAR),
        ("PullDownEnable", UCHAR),
        ("SerNumEnable", UCHAR),
        ("USBVersionEnable", UCHAR),
        ("USBVersion", WORD),
        ("Rev5", UCHAR),
        ("IsoInA", UCHAR),
        ("IsoInB", UCHAR),
        ("IsoOutA", UCHAR),
        ("IsoOutB", UCHAR),
        ("PullDownEnable5", UCHAR),
        ("SerNumEnable5", UCHAR),
        ("USBVersionEnable5", UCHAR),
        ("USBVersion5", WORD),
        ("AIsHighCurrent", UCHAR),
        ("BIsHighCurrent", UCHAR),
        ("IFAIsFifo", UCHAR),
        ("IFAIsFifoTar", UCHAR),
        ("IFAIsFastSer", UCHAR),
        ("AIsVCP", UCHAR),
        ("IFBIsFifo", UCHAR),
        ("IFBIsFifoTar", UCHAR),
        ("IFBIsFastSer", UCHAR),
        ("BIsVCP", UCHAR),
        ("UseExtOsc", UCHAR),
        ("HighDriveIOs", UCHAR),
        ("EndpointSize", UCHAR),
        ("PullDownEnableR", UCHAR),
        ("SerNumEnableR", UCHAR),
        ("InvertTXD", UCHAR),
        ("InvertRXD", UCHAR),
        ("InvertRTS", UCHAR),
        ("InvertCTS", UCHAR),
        ("InvertDTR", UCHAR),
        ("InvertDSR", UCHAR),
        ("InvertDCD", UCHAR),
        ("InvertRI", UCHAR),
        ("Cbus0", UCHAR),
        ("Cbus1", UCHAR),
        ("Cbus2", UCHAR),
        ("Cbus3", UCHAR),
        ("Cbus4", UCHAR),
        ("RIsVCP", UCHAR),
    ]


PFT_PROGRAM_DATA = POINTER(ft_program_data)
FT_PROGRAM_DATA = ft_program_data
# ftd2xx_linux.h 518
FT_EE_Program = _libraries["libftd2xx.so"].FT_EE_Program
FT_EE_Program.restype = FT_STATUS
# FT_EE_Program(ftHandle, pData)
FT_EE_Program.argtypes = [FT_HANDLE, PFT_PROGRAM_DATA]
FT_EE_Program.__doc__ = """FT_STATUS FT_EE_Program(FT_HANDLE ftHandle, PFT_PROGRAM_DATA pData)
ftd2xx_linux.h:518"""
# ftd2xx_linux.h 528
FT_EE_ProgramEx = _libraries["libftd2xx.so"].FT_EE_ProgramEx
FT_EE_ProgramEx.restype = FT_STATUS
# FT_EE_ProgramEx(ftHandle, lpData, Manufacturer, ManufacturerId, Description, SerialNumber)
FT_EE_ProgramEx.argtypes = [FT_HANDLE, PFT_PROGRAM_DATA, STRING, STRING, STRING, STRING]
FT_EE_ProgramEx.__doc__ = """FT_STATUS FT_EE_ProgramEx(FT_HANDLE ftHandle, PFT_PROGRAM_DATA lpData, char * Manufacturer, char * ManufacturerId, char * Description, char * SerialNumber)
ftd2xx_linux.h:528"""
# ftd2xx_linux.h 534
FT_EE_Read = _libraries["libftd2xx.so"].FT_EE_Read
FT_EE_Read.restype = FT_STATUS
# FT_EE_Read(ftHandle, pData)
FT_EE_Read.argtypes = [FT_HANDLE, PFT_PROGRAM_DATA]
FT_EE_Read.__doc__ = """FT_STATUS FT_EE_Read(FT_HANDLE ftHandle, PFT_PROGRAM_DATA pData)
ftd2xx_linux.h:534"""
# ftd2xx_linux.h 544
FT_EE_ReadEx = _libraries["libftd2xx.so"].FT_EE_ReadEx
FT_EE_ReadEx.restype = FT_STATUS
# FT_EE_ReadEx(ftHandle, lpData, Manufacturer, ManufacturerId, Description, SerialNumber)
FT_EE_ReadEx.argtypes = [FT_HANDLE, PFT_PROGRAM_DATA, STRING, STRING, STRING, STRING]
FT_EE_ReadEx.__doc__ = """FT_STATUS FT_EE_ReadEx(FT_HANDLE ftHandle, PFT_PROGRAM_DATA lpData, char * Manufacturer, char * ManufacturerId, char * Description, char * SerialNumber)
ftd2xx_linux.h:544"""
# ftd2xx_linux.h 550
FT_EE_UASize = _libraries["libftd2xx.so"].FT_EE_UASize
FT_EE_UASize.restype = FT_STATUS
# FT_EE_UASize(ftHandle, lpdwSize)
FT_EE_UASize.argtypes = [FT_HANDLE, LPDWORD]
FT_EE_UASize.__doc__ = """FT_STATUS FT_EE_UASize(FT_HANDLE ftHandle, LPDWORD lpdwSize)
ftd2xx_linux.h:550"""
# ftd2xx_linux.h 557
FT_EE_UAWrite = _libraries["libftd2xx.so"].FT_EE_UAWrite
FT_EE_UAWrite.restype = FT_STATUS
# FT_EE_UAWrite(ftHandle, pucData, dwDataLen)
FT_EE_UAWrite.argtypes = [FT_HANDLE, PUCHAR, DWORD]
FT_EE_UAWrite.__doc__ = """FT_STATUS FT_EE_UAWrite(FT_HANDLE ftHandle, PUCHAR pucData, DWORD dwDataLen)
ftd2xx_linux.h:557"""
# ftd2xx_linux.h 565
FT_EE_UARead = _libraries["libftd2xx.so"].FT_EE_UARead
FT_EE_UARead.restype = FT_STATUS
# FT_EE_UARead(ftHandle, pucData, dwDataLen, lpdwBytesRead)
FT_EE_UARead.argtypes = [FT_HANDLE, PUCHAR, DWORD, LPDWORD]
FT_EE_UARead.__doc__ = """FT_STATUS FT_EE_UARead(FT_HANDLE ftHandle, PUCHAR pucData, DWORD dwDataLen, LPDWORD lpdwBytesRead)
ftd2xx_linux.h:565"""
# ftd2xx_linux.h 571
FT_SetLatencyTimer = _libraries["libftd2xx.so"].FT_SetLatencyTimer
FT_SetLatencyTimer.restype = FT_STATUS
# FT_SetLatencyTimer(ftHandle, ucLatency)
FT_SetLatencyTimer.argtypes = [FT_HANDLE, UCHAR]
FT_SetLatencyTimer.__doc__ = """FT_STATUS FT_SetLatencyTimer(FT_HANDLE ftHandle, UCHAR ucLatency)
ftd2xx_linux.h:571"""
# ftd2xx_linux.h 577
FT_GetLatencyTimer = _libraries["libftd2xx.so"].FT_GetLatencyTimer
FT_GetLatencyTimer.restype = FT_STATUS
# FT_GetLatencyTimer(ftHandle, pucLatency)
FT_GetLatencyTimer.argtypes = [FT_HANDLE, PUCHAR]
FT_GetLatencyTimer.__doc__ = """FT_STATUS FT_GetLatencyTimer(FT_HANDLE ftHandle, PUCHAR pucLatency)
ftd2xx_linux.h:577"""
# ftd2xx_linux.h 584
FT_SetBitMode = _libraries["libftd2xx.so"].FT_SetBitMode
FT_SetBitMode.restype = FT_STATUS
# FT_SetBitMode(ftHandle, ucMask, ucEnable)
FT_SetBitMode.argtypes = [FT_HANDLE, UCHAR, UCHAR]
FT_SetBitMode.__doc__ = """FT_STATUS FT_SetBitMode(FT_HANDLE ftHandle, UCHAR ucMask, UCHAR ucEnable)
ftd2xx_linux.h:584"""
# ftd2xx_linux.h 590
FT_GetBitMode = _libraries["libftd2xx.so"].FT_GetBitMode
FT_GetBitMode.restype = FT_STATUS
# FT_GetBitMode(ftHandle, pucMode)
FT_GetBitMode.argtypes = [FT_HANDLE, PUCHAR]
FT_GetBitMode.__doc__ = """FT_STATUS FT_GetBitMode(FT_HANDLE ftHandle, PUCHAR pucMode)
ftd2xx_linux.h:590"""
# ftd2xx_linux.h 597
FT_SetUSBParameters = _libraries["libftd2xx.so"].FT_SetUSBParameters
FT_SetUSBParameters.restype = FT_STATUS
# FT_SetUSBParameters(ftHandle, ulInTransferSize, ulOutTransferSize)
FT_SetUSBParameters.argtypes = [FT_HANDLE, ULONG, ULONG]
FT_SetUSBParameters.__doc__ = """FT_STATUS FT_SetUSBParameters(FT_HANDLE ftHandle, ULONG ulInTransferSize, ULONG ulOutTransferSize)
ftd2xx_linux.h:597"""
# ftd2xx_linux.h 603
FT_SetDeadmanTimeout = _libraries["libftd2xx.so"].FT_SetDeadmanTimeout
FT_SetDeadmanTimeout.restype = FT_STATUS
# FT_SetDeadmanTimeout(ftHandle, ulDeadmanTimeout)
FT_SetDeadmanTimeout.argtypes = [FT_HANDLE, ULONG]
FT_SetDeadmanTimeout.__doc__ = """FT_STATUS FT_SetDeadmanTimeout(FT_HANDLE ftHandle, ULONG ulDeadmanTimeout)
ftd2xx_linux.h:603"""
# ftd2xx_linux.h 613
FT_GetDeviceInfo = _libraries["libftd2xx.so"].FT_GetDeviceInfo
FT_GetDeviceInfo.restype = FT_STATUS
# FT_GetDeviceInfo(ftHandle, lpftDevice, lpdwID, SerialNumber, Description, Dummy)
FT_GetDeviceInfo.argtypes = [
    FT_HANDLE,
    POINTER(FT_DEVICE),
    LPDWORD,
    PCHAR,
    PCHAR,
    LPVOID,
]
FT_GetDeviceInfo.__doc__ = """FT_STATUS FT_GetDeviceInfo(FT_HANDLE ftHandle, FT_DEVICE * lpftDevice, LPDWORD lpdwID, PCHAR SerialNumber, PCHAR Description, LPVOID Dummy)
ftd2xx_linux.h:613"""
# ftd2xx_linux.h 618
FT_StopInTask = _libraries["libftd2xx.so"].FT_StopInTask
FT_StopInTask.restype = FT_STATUS
# FT_StopInTask(ftHandle)
FT_StopInTask.argtypes = [FT_HANDLE]
FT_StopInTask.__doc__ = """FT_STATUS FT_StopInTask(FT_HANDLE ftHandle)
ftd2xx_linux.h:618"""
# ftd2xx_linux.h 623
FT_RestartInTask = _libraries["libftd2xx.so"].FT_RestartInTask
FT_RestartInTask.restype = FT_STATUS
# FT_RestartInTask(ftHandle)
FT_RestartInTask.argtypes = [FT_HANDLE]
FT_RestartInTask.__doc__ = """FT_STATUS FT_RestartInTask(FT_HANDLE ftHandle)
ftd2xx_linux.h:623"""
# ftd2xx_linux.h 629
FT_SetResetPipeRetryCount = _libraries["libftd2xx.so"].FT_SetResetPipeRetryCount
FT_SetResetPipeRetryCount.restype = FT_STATUS
# FT_SetResetPipeRetryCount(ftHandle, dwCount)
FT_SetResetPipeRetryCount.argtypes = [FT_HANDLE, DWORD]
FT_SetResetPipeRetryCount.__doc__ = """FT_STATUS FT_SetResetPipeRetryCount(FT_HANDLE ftHandle, DWORD dwCount)
ftd2xx_linux.h:629"""
# ftd2xx_linux.h 634
FT_ResetPort = _libraries["libftd2xx.so"].FT_ResetPort
FT_ResetPort.restype = FT_STATUS
# FT_ResetPort(ftHandle)
FT_ResetPort.argtypes = [FT_HANDLE]
FT_ResetPort.__doc__ = """FT_STATUS FT_ResetPort(FT_HANDLE ftHandle)
ftd2xx_linux.h:634"""
# ftd2xx_linux.h 639
FT_CyclePort = _libraries["libftd2xx.so"].FT_CyclePort
FT_CyclePort.restype = FT_STATUS
# FT_CyclePort(ftHandle)
FT_CyclePort.argtypes = [FT_HANDLE]
FT_CyclePort.__doc__ = """FT_STATUS FT_CyclePort(FT_HANDLE ftHandle)
ftd2xx_linux.h:639"""
# ftd2xx_linux.h 655
FT_W32_CreateFile = _libraries["libftd2xx.so"].FT_W32_CreateFile
FT_W32_CreateFile.restype = FT_HANDLE
# FT_W32_CreateFile(lpszName, dwAccess, dwShareMode, lpSecurityAttributes, dwCreate, dwAttrsAndFlags, hTemplate)
FT_W32_CreateFile.argtypes = [
    LPCSTR,
    DWORD,
    DWORD,
    LPSECURITY_ATTRIBUTES,
    DWORD,
    DWORD,
    HANDLE,
]
FT_W32_CreateFile.__doc__ = """FT_HANDLE FT_W32_CreateFile(LPCSTR lpszName, DWORD dwAccess, DWORD dwShareMode, LPSECURITY_ATTRIBUTES lpSecurityAttributes, DWORD dwCreate, DWORD dwAttrsAndFlags, HANDLE hTemplate)
ftd2xx_linux.h:655"""
# ftd2xx_linux.h 660
FT_W32_CloseHandle = _libraries["libftd2xx.so"].FT_W32_CloseHandle
FT_W32_CloseHandle.restype = BOOL
# FT_W32_CloseHandle(ftHandle)
FT_W32_CloseHandle.argtypes = [FT_HANDLE]
FT_W32_CloseHandle.__doc__ = """BOOL FT_W32_CloseHandle(FT_HANDLE ftHandle)
ftd2xx_linux.h:660"""
# ftd2xx_linux.h 669
FT_W32_ReadFile = _libraries["libftd2xx.so"].FT_W32_ReadFile
FT_W32_ReadFile.restype = BOOL
# FT_W32_ReadFile(ftHandle, lpBuffer, nBufferSize, lpBytesReturned, lpOverlapped)
FT_W32_ReadFile.argtypes = [FT_HANDLE, LPVOID, DWORD, LPDWORD, LPOVERLAPPED]
FT_W32_ReadFile.__doc__ = """BOOL FT_W32_ReadFile(FT_HANDLE ftHandle, LPVOID lpBuffer, DWORD nBufferSize, LPDWORD lpBytesReturned, LPOVERLAPPED lpOverlapped)
ftd2xx_linux.h:669"""
# ftd2xx_linux.h 678
FT_W32_WriteFile = _libraries["libftd2xx.so"].FT_W32_WriteFile
FT_W32_WriteFile.restype = BOOL
# FT_W32_WriteFile(ftHandle, lpBuffer, nBufferSize, lpBytesWritten, lpOverlapped)
FT_W32_WriteFile.argtypes = [FT_HANDLE, LPVOID, DWORD, LPDWORD, LPOVERLAPPED]
FT_W32_WriteFile.__doc__ = """BOOL FT_W32_WriteFile(FT_HANDLE ftHandle, LPVOID lpBuffer, DWORD nBufferSize, LPDWORD lpBytesWritten, LPOVERLAPPED lpOverlapped)
ftd2xx_linux.h:678"""
# ftd2xx_linux.h 683
FT_W32_GetLastError = _libraries["libftd2xx.so"].FT_W32_GetLastError
FT_W32_GetLastError.restype = DWORD
# FT_W32_GetLastError(ftHandle)
FT_W32_GetLastError.argtypes = [FT_HANDLE]
FT_W32_GetLastError.__doc__ = """DWORD FT_W32_GetLastError(FT_HANDLE ftHandle)
ftd2xx_linux.h:683"""
# ftd2xx_linux.h 691
FT_W32_GetOverlappedResult = _libraries["libftd2xx.so"].FT_W32_GetOverlappedResult
FT_W32_GetOverlappedResult.restype = BOOL
# FT_W32_GetOverlappedResult(ftHandle, lpOverlapped, lpdwBytesTransferred, bWait)
FT_W32_GetOverlappedResult.argtypes = [FT_HANDLE, LPOVERLAPPED, LPDWORD, BOOL]
FT_W32_GetOverlappedResult.__doc__ = """BOOL FT_W32_GetOverlappedResult(FT_HANDLE ftHandle, LPOVERLAPPED lpOverlapped, LPDWORD lpdwBytesTransferred, BOOL bWait)
ftd2xx_linux.h:691"""
# ftd2xx_linux.h 696
FT_W32_CancelIo = _libraries["libftd2xx.so"].FT_W32_CancelIo
FT_W32_CancelIo.restype = BOOL
# FT_W32_CancelIo(ftHandle)
FT_W32_CancelIo.argtypes = [FT_HANDLE]
FT_W32_CancelIo.__doc__ = """BOOL FT_W32_CancelIo(FT_HANDLE ftHandle)
ftd2xx_linux.h:696"""
# ftd2xx_linux.h 702


class _FTCOMSTAT(Structure):
    _fields_ = [
        # ftd2xx_linux.h 702
        ("fCtsHold", DWORD, 1),
        ("fDsrHold", DWORD, 1),
        ("fRlsdHold", DWORD, 1),
        ("fXoffHold", DWORD, 1),
        ("fXoffSent", DWORD, 1),
        ("fEof", DWORD, 1),
        ("fTxim", DWORD, 1),
        ("fReserved", DWORD, 25),
        ("cbInQue", DWORD),
        ("cbOutQue", DWORD),
    ]


LPFTCOMSTAT = POINTER(_FTCOMSTAT)
FTCOMSTAT = _FTCOMSTAT
# ftd2xx_linux.h 715


class _FTDCB(Structure):
    _fields_ = [
        # ftd2xx_linux.h 715
        ("DCBlength", DWORD),
        ("BaudRate", DWORD),
        ("fBinary", DWORD, 1),
        ("fParity", DWORD, 1),
        ("fOutxCtsFlow", DWORD, 1),
        ("fOutxDsrFlow", DWORD, 1),
        ("fDtrControl", DWORD, 2),
        ("fDsrSensitivity", DWORD, 1),
        ("fTXContinueOnXoff", DWORD, 1),
        ("fOutX", DWORD, 1),
        ("fInX", DWORD, 1),
        ("fErrorChar", DWORD, 1),
        ("fNull", DWORD, 1),
        ("fRtsControl", DWORD, 2),
        ("fAbortOnError", DWORD, 1),
        ("fDummy2", DWORD, 17),
        ("wReserved", WORD),
        ("XonLim", WORD),
        ("XoffLim", WORD),
        ("ByteSize", BYTE),
        ("Parity", BYTE),
        ("StopBits", BYTE),
        ("XonChar", c_char),
        ("XoffChar", c_char),
        ("ErrorChar", c_char),
        ("EofChar", c_char),
        ("EvtChar", c_char),
        ("wReserved1", WORD),
    ]


LPFTDCB = POINTER(_FTDCB)
FTDCB = _FTDCB
# ftd2xx_linux.h 746


class _FTTIMEOUTS(Structure):
    _fields_ = [
        # ftd2xx_linux.h 746
        ("ReadIntervalTimeout", DWORD),
        ("ReadTotalTimeoutMultiplier", DWORD),
        ("ReadTotalTimeoutConstant", DWORD),
        ("WriteTotalTimeoutMultiplier", DWORD),
        ("WriteTotalTimeoutConstant", DWORD),
    ]


FTTIMEOUTS = _FTTIMEOUTS
LPFTTIMEOUTS = POINTER(_FTTIMEOUTS)
# ftd2xx_linux.h 758
FT_W32_ClearCommBreak = _libraries["libftd2xx.so"].FT_W32_ClearCommBreak
FT_W32_ClearCommBreak.restype = BOOL
# FT_W32_ClearCommBreak(ftHandle)
FT_W32_ClearCommBreak.argtypes = [FT_HANDLE]
FT_W32_ClearCommBreak.__doc__ = """BOOL FT_W32_ClearCommBreak(FT_HANDLE ftHandle)
ftd2xx_linux.h:758"""
# ftd2xx_linux.h 765
FT_W32_ClearCommError = _libraries["libftd2xx.so"].FT_W32_ClearCommError
FT_W32_ClearCommError.restype = BOOL
# FT_W32_ClearCommError(ftHandle, lpdwErrors, lpftComstat)
FT_W32_ClearCommError.argtypes = [FT_HANDLE, LPDWORD, LPFTCOMSTAT]
FT_W32_ClearCommError.__doc__ = """BOOL FT_W32_ClearCommError(FT_HANDLE ftHandle, LPDWORD lpdwErrors, LPFTCOMSTAT lpftComstat)
ftd2xx_linux.h:765"""
# ftd2xx_linux.h 771
FT_W32_EscapeCommFunction = _libraries["libftd2xx.so"].FT_W32_EscapeCommFunction
FT_W32_EscapeCommFunction.restype = BOOL
# FT_W32_EscapeCommFunction(ftHandle, dwFunc)
FT_W32_EscapeCommFunction.argtypes = [FT_HANDLE, DWORD]
FT_W32_EscapeCommFunction.__doc__ = """BOOL FT_W32_EscapeCommFunction(FT_HANDLE ftHandle, DWORD dwFunc)
ftd2xx_linux.h:771"""
# ftd2xx_linux.h 777
FT_W32_GetCommModemStatus = _libraries["libftd2xx.so"].FT_W32_GetCommModemStatus
FT_W32_GetCommModemStatus.restype = BOOL
# FT_W32_GetCommModemStatus(ftHandle, lpdwModemStatus)
FT_W32_GetCommModemStatus.argtypes = [FT_HANDLE, LPDWORD]
FT_W32_GetCommModemStatus.__doc__ = """BOOL FT_W32_GetCommModemStatus(FT_HANDLE ftHandle, LPDWORD lpdwModemStatus)
ftd2xx_linux.h:777"""
# ftd2xx_linux.h 783
FT_W32_GetCommState = _libraries["libftd2xx.so"].FT_W32_GetCommState
FT_W32_GetCommState.restype = BOOL
# FT_W32_GetCommState(ftHandle, lpftDcb)
FT_W32_GetCommState.argtypes = [FT_HANDLE, LPFTDCB]
FT_W32_GetCommState.__doc__ = """BOOL FT_W32_GetCommState(FT_HANDLE ftHandle, LPFTDCB lpftDcb)
ftd2xx_linux.h:783"""
# ftd2xx_linux.h 789
FT_W32_GetCommTimeouts = _libraries["libftd2xx.so"].FT_W32_GetCommTimeouts
FT_W32_GetCommTimeouts.restype = BOOL
# FT_W32_GetCommTimeouts(ftHandle, pTimeouts)
FT_W32_GetCommTimeouts.argtypes = [FT_HANDLE, POINTER(FTTIMEOUTS)]
FT_W32_GetCommTimeouts.__doc__ = """BOOL FT_W32_GetCommTimeouts(FT_HANDLE ftHandle, FTTIMEOUTS * pTimeouts)
ftd2xx_linux.h:789"""
# ftd2xx_linux.h 795
FT_W32_PurgeComm = _libraries["libftd2xx.so"].FT_W32_PurgeComm
FT_W32_PurgeComm.restype = BOOL
# FT_W32_PurgeComm(ftHandle, dwMask)
FT_W32_PurgeComm.argtypes = [FT_HANDLE, DWORD]
FT_W32_PurgeComm.__doc__ = """BOOL FT_W32_PurgeComm(FT_HANDLE ftHandle, DWORD dwMask)
ftd2xx_linux.h:795"""
# ftd2xx_linux.h 800
FT_W32_SetCommBreak = _libraries["libftd2xx.so"].FT_W32_SetCommBreak
FT_W32_SetCommBreak.restype = BOOL
# FT_W32_SetCommBreak(ftHandle)
FT_W32_SetCommBreak.argtypes = [FT_HANDLE]
FT_W32_SetCommBreak.__doc__ = """BOOL FT_W32_SetCommBreak(FT_HANDLE ftHandle)
ftd2xx_linux.h:800"""
# ftd2xx_linux.h 806
FT_W32_SetCommMask = _libraries["libftd2xx.so"].FT_W32_SetCommMask
FT_W32_SetCommMask.restype = BOOL
# FT_W32_SetCommMask(ftHandle, ulEventMask)
FT_W32_SetCommMask.argtypes = [FT_HANDLE, ULONG]
FT_W32_SetCommMask.__doc__ = """BOOL FT_W32_SetCommMask(FT_HANDLE ftHandle, ULONG ulEventMask)
ftd2xx_linux.h:806"""
# ftd2xx_linux.h 812
FT_W32_SetCommState = _libraries["libftd2xx.so"].FT_W32_SetCommState
FT_W32_SetCommState.restype = BOOL
# FT_W32_SetCommState(ftHandle, lpftDcb)
FT_W32_SetCommState.argtypes = [FT_HANDLE, LPFTDCB]
FT_W32_SetCommState.__doc__ = """BOOL FT_W32_SetCommState(FT_HANDLE ftHandle, LPFTDCB lpftDcb)
ftd2xx_linux.h:812"""
# ftd2xx_linux.h 818
FT_W32_SetCommTimeouts = _libraries["libftd2xx.so"].FT_W32_SetCommTimeouts
FT_W32_SetCommTimeouts.restype = BOOL
# FT_W32_SetCommTimeouts(ftHandle, pTimeouts)
FT_W32_SetCommTimeouts.argtypes = [FT_HANDLE, POINTER(FTTIMEOUTS)]
FT_W32_SetCommTimeouts.__doc__ = """BOOL FT_W32_SetCommTimeouts(FT_HANDLE ftHandle, FTTIMEOUTS * pTimeouts)
ftd2xx_linux.h:818"""
# ftd2xx_linux.h 825
FT_W32_SetupComm = _libraries["libftd2xx.so"].FT_W32_SetupComm
FT_W32_SetupComm.restype = BOOL
# FT_W32_SetupComm(ftHandle, dwReadBufferSize, dwWriteBufferSize)
FT_W32_SetupComm.argtypes = [FT_HANDLE, DWORD, DWORD]
FT_W32_SetupComm.__doc__ = """BOOL FT_W32_SetupComm(FT_HANDLE ftHandle, DWORD dwReadBufferSize, DWORD dwWriteBufferSize)
ftd2xx_linux.h:825"""
# ftd2xx_linux.h 832
FT_W32_WaitCommEvent = _libraries["libftd2xx.so"].FT_W32_WaitCommEvent
FT_W32_WaitCommEvent.restype = BOOL
# FT_W32_WaitCommEvent(ftHandle, pulEvent, lpOverlapped)
FT_W32_WaitCommEvent.argtypes = [FT_HANDLE, PULONG, LPOVERLAPPED]
FT_W32_WaitCommEvent.__doc__ = """BOOL FT_W32_WaitCommEvent(FT_HANDLE ftHandle, PULONG pulEvent, LPOVERLAPPED lpOverlapped)
ftd2xx_linux.h:832"""
# ftd2xx_linux.h 838


class _ft_device_list_info_node(Structure):
    _fields_ = [
        # ftd2xx_linux.h 838
        ("Flags", ULONG),
        ("Type", ULONG),
        ("ID", ULONG),
        ("LocId", DWORD),
        ("SerialNumber", c_char * 16),
        ("Description", c_char * 64),
        ("ftHandle", FT_HANDLE),
    ]


FT_DEVICE_LIST_INFO_NODE = _ft_device_list_info_node
# ftd2xx_linux.h 851
FT_CreateDeviceInfoList = _libraries["libftd2xx.so"].FT_CreateDeviceInfoList
FT_CreateDeviceInfoList.restype = FT_STATUS
# FT_CreateDeviceInfoList(lpdwNumDevs)
FT_CreateDeviceInfoList.argtypes = [LPDWORD]
FT_CreateDeviceInfoList.__doc__ = """FT_STATUS FT_CreateDeviceInfoList(LPDWORD lpdwNumDevs)
ftd2xx_linux.h:851"""
# ftd2xx_linux.h 857
FT_GetDeviceInfoList = _libraries["libftd2xx.so"].FT_GetDeviceInfoList
FT_GetDeviceInfoList.restype = FT_STATUS
# FT_GetDeviceInfoList(pDest, lpdwNumDevs)
FT_GetDeviceInfoList.argtypes = [POINTER(FT_DEVICE_LIST_INFO_NODE), LPDWORD]
FT_GetDeviceInfoList.__doc__ = """FT_STATUS FT_GetDeviceInfoList(FT_DEVICE_LIST_INFO_NODE * pDest, LPDWORD lpdwNumDevs)
ftd2xx_linux.h:857"""
# ftd2xx_linux.h 869
FT_GetDeviceInfoDetail = _libraries["libftd2xx.so"].FT_GetDeviceInfoDetail
FT_GetDeviceInfoDetail.restype = FT_STATUS
# FT_GetDeviceInfoDetail(dwIndex, lpdwFlags, lpdwType, lpdwID, lpdwLocId, lpSerialNumber, lpDescription, pftHandle)
FT_GetDeviceInfoDetail.argtypes = [
    DWORD,
    LPDWORD,
    LPDWORD,
    LPDWORD,
    LPDWORD,
    LPVOID,
    LPVOID,
    POINTER(FT_HANDLE),
]
FT_GetDeviceInfoDetail.__doc__ = """FT_STATUS FT_GetDeviceInfoDetail(DWORD dwIndex, LPDWORD lpdwFlags, LPDWORD lpdwType, LPDWORD lpdwID, LPDWORD lpdwLocId, LPVOID lpSerialNumber, LPVOID lpDescription, FT_HANDLE * pftHandle)
ftd2xx_linux.h:869"""
# ftd2xx_linux.h 875
FT_GetDriverVersion = _libraries["libftd2xx.so"].FT_GetDriverVersion
FT_GetDriverVersion.restype = FT_STATUS
# FT_GetDriverVersion(ftHandle, lpdwVersion)
FT_GetDriverVersion.argtypes = [FT_HANDLE, LPDWORD]
FT_GetDriverVersion.__doc__ = """FT_STATUS FT_GetDriverVersion(FT_HANDLE ftHandle, LPDWORD lpdwVersion)
ftd2xx_linux.h:875"""
# ftd2xx_linux.h 880
FT_GetLibraryVersion = _libraries["libftd2xx.so"].FT_GetLibraryVersion
FT_GetLibraryVersion.restype = FT_STATUS
# FT_GetLibraryVersion(lpdwVersion)
FT_GetLibraryVersion.argtypes = [LPDWORD]
FT_GetLibraryVersion.__doc__ = """FT_STATUS FT_GetLibraryVersion(LPDWORD lpdwVersion)
ftd2xx_linux.h:880"""

__all__ = [
    "FT_SetVIDPID",  # Linux/Darwin only
    "FT_GetVIDPID",  # Linux/Darwin only
    "FT_CreateDeviceInfoList",
    "FT_GetDeviceInfoList",
    "FT_GetDeviceInfoDetail",
    "FT_ListDevices",
    "FT_Open",
    "FT_OpenEx",
    "FT_Close",
    "FT_Read",
    "FT_Write",
    "FT_SetBaudRate",
    "FT_SetDivisor",
    "FT_SetDataCharacteristics",
    "FT_SetTimeouts",
    "FT_SetFlowControl",
    "FT_SetDtr",
    "FT_ClrDtr",
    "FT_SetRts",
    "FT_ClrRts",
    "FT_GetModemStatus",
    "FT_GetQueueStatus",
    "FT_GetDeviceInfo",
    # "FT_GetDriverVersion",  # Windows only
    # "FT_GetLibraryVersion",  # Windows only
    # "FT_GetComPortNumber",  # Windows only
    "FT_GetStatus",
    "FT_SetEventNotification",
    "FT_SetChars",
    "FT_SetBreakOn",
    "FT_SetBreakOff",
    "FT_Purge",
    "FT_ResetDevice",
    # "FT_ResetPort",  # Windows only
    # "FT_CyclePort",  # Windows only
    # "FT_Rescan", # Windows only, not implimented yet
    # "FT_Reload",  # Windows only, not implimented yet
    "FT_SetResetPipeRetryCount",  # Windows only
    "FT_StopInTask",
    "FT_RestartInTask",
    "FT_SetDeadmanTimeout",
    "FT_IoCtl",  # Undocumented
    "FT_SetWaitMask",  # Undocumented
    "FT_WaitOnMask",  # Undocumented
    "FT_ReadEE",
    "FT_WriteEE",
    "FT_EraseEE",
    "FT_EE_Read",
    "FT_EE_ReadEx",
    "FT_EE_Program",
    "FT_EE_ProgramEx",
    "FT_EE_UASize",
    "FT_EE_UARead",
    "FT_EE_UAWrite",
    # "FT_EEPROM_Read",  # Windows XP or later, not implimented
    # "FT_EEPROM_Program",  # Windows XP or later, not implimented
]

# Extended API (does not apply to FT8U232AM or FT8U245AM devices)
__all__ += [
    "FT_SetLatencyTimer",
    "FT_GetLatencyTimer",
    "FT_SetBitMode",
    "FT_GetBitMode",
    "FT_SetUSBParameters",
]

# Win32 API (cross-platform)
__all__ += [
    "FT_W32_CreateFile",
    "FT_W32_CloseHandle",
    "FT_W32_ReadFile",
    "FT_W32_WriteFile",
    "FT_W32_GetOverlappedResult",
    "FT_W32_EscapeCommFunction",
    "FT_W32_GetCommModemStatus",
    "FT_W32_SetupComm",
    "FT_W32_SetCommState",
    "FT_W32_GetCommState",
    "FT_W32_SetCommTimeouts",
    "FT_W32_GetCommTimeouts",
    "FT_W32_SetCommBreak",
    "FT_W32_ClearCommBreak",
    "FT_W32_SetCommMask",
    # "FT_W32_GetCommMask",  # Not implimented yet
    "FT_W32_WaitCommEvent",
    "FT_W32_PurgeComm",
    "FT_W32_GetLastError",
    "FT_W32_ClearCommError",
]

# Data types
__all__ += [
    "FT_HANDLE",
    "ft_program_data",
    "BOOL",
    "BYTE",
    "DWORD",
    "HANDLE",
    "LPCSTR",
    "ULONG",
    "WORD",
    "USHORT",
    "PCHAR",
    "LPWORD",
    "PULONG",
    "LPVOID",
    "STRING",
    "UCHAR",
    "PUCHAR",
    "PVOID",
    "LPTSTR",
    "LPDWORD",
    "VOID",
    "ULONGLONG",
]

# What are these?
__all__ += [
    "FT_W32_CancelIo",
    "FT_GetEventStatus",
]



"""
Control FTDI USB chips.

Use :any:`ftd2xx.open` or :any:`ftd2xx.openEx`.
:example:
    with open(0) as device:
        device.write(b"Hello World!")
"""

import sys

try:
    from _version import (
        __version__ as __version__,
        __version_tuple__ as __version_tuple__,
    )
except ImportError:
    __version__ = "unknown"
    __version_tuple__ = (0, 0, 0, "unknown", "unknown")

from .ftd2xx import (
    FTD2XX,
    DeviceError,
    call_ft,
    createDeviceInfoList,
    ft_program_data,
    getDeviceInfoDetail,
    getLibraryVersion,
    listDevices,
    open,
    openEx,
)

__all__ = [
    "call_ft",
    "listDevices",
    "getLibraryVersion",
    "createDeviceInfoList",
    "getDeviceInfoDetail",
    "open",
    "openEx",
    "FTD2XX",
    "DeviceError",
    "ft_program_data",
]
if sys.platform == "win32":
    from .ftd2xx import w32CreateFile

    __all__ += ["w32CreateFile"]
else:
    from .ftd2xx import getVIDPID, setVIDPID

    __all__ += ["getVIDPID", "setVIDPID"]



"""
Module for accessing functions from FTD2XX in an easier to use
_pythonic_ way. For full documentation please refer to the FTDI
Programming Guide. This module is based on Pablo Bleyers d2xx module,
except this uses ctypes instead of an extension approach.
"""

from __future__ import annotations

import ctypes as c
import logging
import sys
from types import TracebackType
from typing import Any, Callable, ContextManager, TypedDict

from . import defines

if sys.platform == "win32":
    from . import _ftd2xx as _ft
elif sys.platform.startswith("linux"):
    from . import _ftd2xx_linux as _ft
elif sys.platform == "darwin":
    from . import _ftd2xx_darwin as _ft
else:
    raise Exception("Unknown platform")


ft_program_data = _ft.ft_program_data

LOGGER = logging.getLogger("ftd2xx")


class DeviceError(Exception):
    """Exception class for status messages"""

    def __init__(self, message: int | Any):
        super().__init__()
        if isinstance(message, int):
            self.message = defines.Status(message).name
        else:
            self.message = str(message)

    def __str__(self):
        return self.message

    def __reduce__(self):
        return type(self), (self.message,)


class DeviceInfoDetail(TypedDict):
    index: int
    flags: int
    type: int
    id: int
    location: int
    serial: bytes
    description: bytes
    handle: _ft.FT_HANDLE


class DeviceInfo(TypedDict):
    type: int
    id: int
    description: bytes
    serial: bytes


class ProgramData(TypedDict, total=False):
    Signature1: _ft.DWORD | int
    Signature2: _ft.DWORD | int
    Version: _ft.DWORD | int
    VendorId: _ft.WORD | int
    ProductId: _ft.WORD | int
    Manufacturer: _ft.STRING | int
    ManufacturerId: _ft.STRING | int
    Description: _ft.STRING | int
    SerialNumber: _ft.STRING | int
    MaxPower: _ft.WORD | int
    PnP: _ft.WORD | int
    SelfPowered: _ft.WORD | int
    RemoteWakeup: _ft.WORD | int
    Rev4: _ft.UCHAR | int
    IsoIn: _ft.UCHAR | int
    IsoOut: _ft.UCHAR | int
    PullDownEnable: _ft.UCHAR | int
    SerNumEnable: _ft.UCHAR | int
    USBVersionEnable: _ft.UCHAR | int
    USBVersion: _ft.WORD | int
    Rev5: _ft.UCHAR | int
    IsoInA: _ft.UCHAR | int
    IsoInB: _ft.UCHAR | int
    IsoOutA: _ft.UCHAR | int
    IsoOutB: _ft.UCHAR | int
    PullDownEnable5: _ft.UCHAR | int
    SerNumEnable5: _ft.UCHAR | int
    USBVersionEnable5: _ft.UCHAR | int
    USBVersion5: _ft.WORD | int
    AIsHighCurrent: _ft.UCHAR | int
    BIsHighCurrent: _ft.UCHAR | int
    IFAIsFifo: _ft.UCHAR | int
    IFAIsFifoTar: _ft.UCHAR | int
    IFAIsFastSer: _ft.UCHAR | int
    AIsVCP: _ft.UCHAR | int
    IFBIsFifo: _ft.UCHAR | int
    IFBIsFifoTar: _ft.UCHAR | int
    IFBIsFastSer: _ft.UCHAR | int
    BIsVCP: _ft.UCHAR | int
    UseExtOsc: _ft.UCHAR | int
    HighDriveIOs: _ft.UCHAR | int
    EndpointSize: _ft.UCHAR | int
    PullDownEnableR: _ft.UCHAR | int
    SerNumEnableR: _ft.UCHAR | int
    InvertTXD: _ft.UCHAR | int
    InvertRXD: _ft.UCHAR | int
    InvertRTS: _ft.UCHAR | int
    InvertCTS: _ft.UCHAR | int
    InvertDTR: _ft.UCHAR | int
    InvertDSR: _ft.UCHAR | int
    InvertDCD: _ft.UCHAR | int
    InvertRI: _ft.UCHAR | int
    Cbus0: _ft.UCHAR | int
    Cbus1: _ft.UCHAR | int
    Cbus2: _ft.UCHAR | int
    Cbus3: _ft.UCHAR | int
    Cbus4: _ft.UCHAR | int
    RIsVCP: _ft.UCHAR | int
    PullDownEnable7: _ft.UCHAR | int
    SerNumEnable7: _ft.UCHAR | int
    ALSlowSlew: _ft.UCHAR | int
    ALSchmittInput: _ft.UCHAR | int
    ALDriveCurrent: _ft.UCHAR | int
    AHSlowSlew: _ft.UCHAR | int
    AHSchmittInput: _ft.UCHAR | int
    AHDriveCurrent: _ft.UCHAR | int
    BLSlowSlew: _ft.UCHAR | int
    BLSchmittInput: _ft.UCHAR | int
    BLDriveCurrent: _ft.UCHAR | int
    BHSlowSlew: _ft.UCHAR | int
    BHSchmittInput: _ft.UCHAR | int
    BHDriveCurrent: _ft.UCHAR | int
    IFAIsFifo7: _ft.UCHAR | int
    IFAIsFifoTar7: _ft.UCHAR | int
    IFAIsFastSer7: _ft.UCHAR | int
    AIsVCP7: _ft.UCHAR | int
    IFBIsFifo7: _ft.UCHAR | int
    IFBIsFifoTar7: _ft.UCHAR | int
    IFBIsFastSer7: _ft.UCHAR | int
    BIsVCP7: _ft.UCHAR | int
    PowerSaveEnable: _ft.UCHAR | int
    PullDownEnable8: _ft.UCHAR | int
    SerNumEnable8: _ft.UCHAR | int
    ASlowSlew: _ft.UCHAR | int
    ASchmittInput: _ft.UCHAR | int
    ADriveCurrent: _ft.UCHAR | int
    BSlowSlew: _ft.UCHAR | int
    BSchmittInput: _ft.UCHAR | int
    BDriveCurrent: _ft.UCHAR | int
    CSlowSlew: _ft.UCHAR | int
    CSchmittInput: _ft.UCHAR | int
    CDriveCurrent: _ft.UCHAR | int
    DSlowSlew: _ft.UCHAR | int
    DSchmittInput: _ft.UCHAR | int
    DDriveCurrent: _ft.UCHAR | int
    ARIIsTXDEN: _ft.UCHAR | int
    BRIIsTXDEN: _ft.UCHAR | int
    CRIIsTXDEN: _ft.UCHAR | int
    DRIIsTXDEN: _ft.UCHAR | int
    AIsVCP8: _ft.UCHAR | int
    BIsVCP8: _ft.UCHAR | int
    CIsVCP8: _ft.UCHAR | int
    DIsVCP8: _ft.UCHAR | int


def call_ft(function: Callable, *args):
    """Call an FTDI function and check the status. Raise exception on error"""
    status = function(*args)
    if status != defines.Status.OK:
        raise DeviceError(status)


def listDevices(flags: int = 0) -> list[bytes] | None:
    """Return a list of serial numbers(default), descriptions or
    locations (Windows only) of the connected FTDI devices depending on value
    of flags"""
    n = _ft.DWORD()
    call_ft(_ft.FT_ListDevices, c.byref(n), None, _ft.DWORD(defines.LIST_NUMBER_ONLY))
    devcount = n.value
    LOGGER.debug("Found %i devices", devcount)
    if devcount:
        # since ctypes has no pointer arithmetic.
        bd = [
            c.create_string_buffer(defines.MAX_DESCRIPTION_SIZE)
            for i in range(devcount)
        ]

        # array of pointers to those strings, initially all NULL
        ba = (c.c_char_p * (devcount + 1))(*[c.addressof(x) for x in bd], None)
        # for i in range(devcount):
        #     ba[i] = c.c_char_p(bd[i])
        call_ft(_ft.FT_ListDevices, ba, c.byref(n), _ft.DWORD(defines.LIST_ALL | flags))
        return [res for res in ba[:devcount]]

    return None


def getLibraryVersion() -> int:
    """Return a long representing library version"""
    m = _ft.DWORD()
    call_ft(_ft.FT_GetLibraryVersion, c.byref(m))
    return m.value


def createDeviceInfoList() -> int:
    """Create the internal device info list and return number of entries"""
    m = _ft.DWORD()
    call_ft(_ft.FT_CreateDeviceInfoList, c.byref(m))
    return m.value


def getDeviceInfoDetail(devnum: int = 0, update: bool = True) -> DeviceInfoDetail:
    """Get an entry from the internal device info list. Set update to
    False to avoid a slow call to createDeviceInfoList."""
    flags = _ft.DWORD()
    typ = _ft.DWORD()
    dev_id = _ft.DWORD()
    location = _ft.DWORD()
    handle = _ft.FT_HANDLE()
    name = c.create_string_buffer(defines.MAX_DESCRIPTION_SIZE)
    description = c.create_string_buffer(defines.MAX_DESCRIPTION_SIZE)
    # createDeviceInfoList is slow, only run if update is True
    if update:
        createDeviceInfoList()
    call_ft(
        _ft.FT_GetDeviceInfoDetail,
        _ft.DWORD(devnum),
        c.byref(flags),
        c.byref(typ),
        c.byref(dev_id),
        c.byref(location),
        name,
        description,
        c.byref(handle),
    )
    return {
        "index": devnum,
        "flags": flags.value,
        "type": typ.value,
        "id": dev_id.value,
        "location": location.value,
        "serial": name.value,
        "description": description.value,
        "handle": handle,
    }


def open(dev: int = 0, update: bool = True) -> FTD2XX:
    """Open a handle to a usb device by index and return an FTD2XX instance for
    it. Set update to False to avoid a slow call to createDeviceInfoList.

    Args:
        dev (int): The device number to open.
        update (bool): Set to False to disable automatic call to createDeviceInfoList.

    Raises:
        DeviceError: If the device cannot be opened.

    Returns:
        An instance of the FTD2XX class if successful. Use it as a context manager.

    Example:
        with open(0, False) as dev:
            dev.write(b"Hello World")
    """
    h = _ft.FT_HANDLE()
    call_ft(_ft.FT_Open, dev, c.byref(h))
    return FTD2XX(h, update=update)


def openEx(
    id_str: bytes, flags: int = defines.OPEN_BY_SERIAL_NUMBER, update: bool = True
) -> FTD2XX:
    """Open a handle to a usb device by serial number(default), description or
    location(Windows only) depending on value of flags and return an FTD2XX
    instance for it. Set update to False to avoid a slow call to createDeviceInfoList.

    Args:
        id_str (bytes): The ID string from listDevices.
        flags (int): Flag (consult D2XX Guide). Defaults to OPEN_BY_SERIAL_NUMBER.
        update (bool): Set to False to disable automatic call to createDeviceInfoList.

    Raises:
        DeviceError: If the device cannot be opened.

    Returns:
        An instance of the FTD2XX class if successful. Use it as a context manager.

    Example:
        with openEx(b"MyDevice", update=False) as dev:
            dev.write(b"Hello World")

    """
    h = _ft.FT_HANDLE()
    call_ft(_ft.FT_OpenEx, id_str, _ft.DWORD(flags), c.byref(h))
    return FTD2XX(h, update=update)


if sys.platform == "win32":
    from win32con import GENERIC_READ, GENERIC_WRITE, OPEN_EXISTING

    def w32CreateFile(
        name: bytes,
        access: int = GENERIC_READ | GENERIC_WRITE,
        flags: int = defines.OPEN_BY_SERIAL_NUMBER,
    ):
        return FTD2XX(
            _ft.FT_W32_CreateFile(
                _ft.STRING(name),
                _ft.DWORD(access),
                _ft.DWORD(0),
                None,
                _ft.DWORD(OPEN_EXISTING),
                _ft.DWORD(flags),
                _ft.HANDLE(0),
            )
        )

else:

    def getVIDPID() -> tuple[int, int]:
        """Linux only. Get the VID and PID of the device"""
        vid = _ft.DWORD()
        pid = _ft.DWORD()
        call_ft(_ft.FT_GetVIDPID, c.byref(vid), c.byref(pid))
        return (vid.value, pid.value)

    def setVIDPID(vid, pid):
        """Linux only. Set the VID and PID of the device"""
        call_ft(_ft.FT_SetVIDPID, _ft.DWORD(vid), _ft.DWORD(pid))
        return None


class FTD2XX(ContextManager["FTD2XX"]):
    """Class for communicating with an FTDI device

    Use :any:`open` or :any:`openEx` to create an instance of this class.
    """

    handle: _ft.FT_HANDLE
    status: int

    def __init__(self, handle: _ft.FT_HANDLE, update: bool = True):
        """Create an instance of the FTD2XX class with the given device handle
        and populate the device info in the instance dictionary.

        Args:
            update (bool): Set False to disable automatic (slow) call to
                            createDeviceInfoList

        """
        self.handle = handle
        self.status = 1
        # createDeviceInfoList is slow, only run if update is True
        if update:
            createDeviceInfoList()
        self.__dict__.update(self.getDeviceInfo())

    def close(self) -> None:
        """Close the device handle"""
        call_ft(_ft.FT_Close, self.handle)
        self.status = 0

    def read(self, nchars: int, raw: bool = True) -> bytes:
        """Read up to nchars bytes of data from the device. Can return fewer if
        timedout. Use getQueueStatus to find how many bytes are available"""
        b_read = _ft.DWORD()
        b = c.create_string_buffer(nchars)
        call_ft(_ft.FT_Read, self.handle, b, nchars, c.byref(b_read))
        return b.raw[: b_read.value] if raw else b.value[: b_read.value]

    def write(self, data: bytes):
        """Send the data to the device. Data must be a string representing the
        bytes to be sent"""
        w = _ft.DWORD()
        call_ft(_ft.FT_Write, self.handle, data, len(data), c.byref(w))
        return w.value

    def ioctl(self):
        """Not implemented"""
        raise NotImplementedError

    def setBaudRate(self, baud: int) -> None:
        """Set the baud rate"""
        call_ft(_ft.FT_SetBaudRate, self.handle, _ft.DWORD(baud))

    def setDivisor(self, div: int):
        """Set the clock divider. The clock will be set to 6e6/(div + 1)."""
        call_ft(_ft.FT_SetDivisor, self.handle, _ft.USHORT(div))

    def setDataCharacteristics(self, wordlen: int, stopbits: int, parity: int):
        """Set the data characteristics for UART"""
        call_ft(
            _ft.FT_SetDataCharacteristics,
            self.handle,
            _ft.UCHAR(wordlen),
            _ft.UCHAR(stopbits),
            _ft.UCHAR(parity),
        )

    def setFlowControl(self, flowcontrol: int, xon: int = -1, xoff: int = -1):
        """Set the flow control for UART"""
        if flowcontrol == defines.FLOW_XON_XOFF and (xon == -1 or xoff == -1):
            raise ValueError
        call_ft(
            _ft.FT_SetFlowControl,
            self.handle,
            _ft.USHORT(flowcontrol),
            _ft.UCHAR(xon),
            _ft.UCHAR(xoff),
        )

    def resetDevice(self):
        """Reset the device"""
        call_ft(_ft.FT_ResetDevice, self.handle)

    def setDtr(self):
        """Set the DTR (Data Terminal Ready) signal of the FTDI device."""
        call_ft(_ft.FT_SetDtr, self.handle)

    def clrDtr(self):
        """Clear the DTR signal of the FTDI device."""
        call_ft(_ft.FT_ClrDtr, self.handle)

    def setRts(self):
        """Set the RTS (Request To Send) signal of the FTDI device."""
        call_ft(_ft.FT_SetRts, self.handle)

    def clrRts(self):
        """Clear the RTS signal of the FTDI device."""
        call_ft(_ft.FT_ClrRts, self.handle)

    def getModemStatus(self) -> defines.ModemStatus:
        """Get the modem status of the FTDI device."""
        m = _ft.DWORD()
        call_ft(_ft.FT_GetModemStatus, self.handle, c.byref(m))
        return defines.ModemStatus(m.value & 0xFFFF)

    def setChars(self, evch: int, evch_en: int, erch: int, erch_en: int):
        """Set the event and error characters for UART"""
        call_ft(
            _ft.FT_SetChars,
            self.handle,
            _ft.UCHAR(evch),
            _ft.UCHAR(evch_en),
            _ft.UCHAR(erch),
            _ft.UCHAR(erch_en),
        )

    def purge(self, mask: int = 0):
        """Purge the receive and/or transmit buffers"""
        if not mask:
            mask = defines.PURGE_RX | defines.PURGE_TX
        call_ft(_ft.FT_Purge, self.handle, _ft.DWORD(mask))

    def setTimeouts(self, read: int, write: int):
        """Set the read and write timeouts in milliseconds"""
        call_ft(_ft.FT_SetTimeouts, self.handle, _ft.DWORD(read), _ft.DWORD(write))

    def setDeadmanTimeout(self, timeout: int):
        """Set the deadman timeout in milliseconds"""
        call_ft(_ft.FT_SetDeadmanTimeout, self.handle, _ft.DWORD(timeout))

    def getQueueStatus(self) -> int:
        """Get number of bytes in receive queue."""
        rxQAmount = _ft.DWORD()
        call_ft(_ft.FT_GetQueueStatus, self.handle, c.byref(rxQAmount))
        return rxQAmount.value

    def setEventNotification(self, evtmask: int, evthandle):
        call_ft(
            _ft.FT_SetEventNotification,
            self.handle,
            _ft.DWORD(evtmask),
            _ft.HANDLE(evthandle),
        )

    def getStatus(self):
        """Return a 3-tuple of rx queue bytes, tx queue bytes and event
        status"""
        rxQAmount = _ft.DWORD()
        txQAmount = _ft.DWORD()
        evtStatus = _ft.DWORD()
        call_ft(
            _ft.FT_GetStatus,
            self.handle,
            c.byref(rxQAmount),
            c.byref(txQAmount),
            c.byref(evtStatus),
        )
        return (rxQAmount.value, txQAmount.value, evtStatus.value)

    def setBreakOn(self):
        call_ft(_ft.FT_SetBreakOn, self.handle)

    def setBreakOff(self):
        call_ft(_ft.FT_SetBreakOff, self.handle)

    def setWaitMask(self, mask: int):
        call_ft(_ft.FT_SetWaitMask, self.handle, _ft.DWORD(mask))

    def waitOnMask(self):
        mask = _ft.DWORD()
        call_ft(_ft.FT_WaitOnMask, self.handle, c.byref(mask))
        return mask.value

    def getEventStatus(self):
        evtStatus = _ft.DWORD()
        call_ft(_ft.FT_GetEventStatus, self.handle, c.byref(evtStatus))
        return evtStatus.value

    def setLatencyTimer(self, latency: int):
        call_ft(_ft.FT_SetLatencyTimer, self.handle, _ft.UCHAR(latency))

    def getLatencyTimer(self) -> int:
        latency = _ft.UCHAR()
        call_ft(_ft.FT_GetLatencyTimer, self.handle, c.byref(latency))
        return latency.value

    def setBitMode(self, mask: int, enable: int):
        call_ft(_ft.FT_SetBitMode, self.handle, _ft.UCHAR(mask), _ft.UCHAR(enable))

    def getBitMode(self) -> int:
        mask = _ft.UCHAR()
        call_ft(_ft.FT_GetBitMode, self.handle, c.byref(mask))
        return mask.value

    def setUSBParameters(self, in_tx_size: int, out_tx_size: int = 0):
        """Set the USB request transfer sizes"""
        call_ft(
            _ft.FT_SetUSBParameters,
            self.handle,
            _ft.ULONG(in_tx_size),
            _ft.ULONG(out_tx_size),
        )

    def getDeviceInfo(self) -> DeviceInfo:
        """Returns a dictionary describing the device."""
        deviceType = _ft.DWORD()
        deviceId = _ft.DWORD()
        desc = c.create_string_buffer(defines.MAX_DESCRIPTION_SIZE)
        serial = c.create_string_buffer(defines.MAX_DESCRIPTION_SIZE)

        call_ft(
            _ft.FT_GetDeviceInfo,
            self.handle,
            c.byref(deviceType),
            c.byref(deviceId),
            serial,
            desc,
            None,
        )
        return {
            "type": deviceType.value,
            "id": deviceId.value,
            "description": desc.value,
            "serial": serial.value,
        }

    def stopInTask(self):
        call_ft(_ft.FT_StopInTask, self.handle)

    def restartInTask(self):
        call_ft(_ft.FT_RestartInTask, self.handle)

    def setRestPipeRetryCount(self, count):
        call_ft(_ft.FT_SetResetPipeRetryCount, self.handle, _ft.DWORD(count))

    def resetPort(self):
        call_ft(_ft.FT_ResetPort, self.handle)

    def cyclePort(self):
        call_ft(_ft.FT_CyclePort, self.handle)

    def getDriverVersion(self) -> int:
        driver = _ft.DWORD()
        call_ft(_ft.FT_GetDriverVersion, self.handle, c.byref(driver))
        return driver.value

    def getComPortNumber(self) -> int:
        """Return a long representing the COM port number"""
        m = _ft.LONG()
        try:
            call_ft(_ft.FT_GetComPortNumber, self.handle, c.byref(m))
        except AttributeError as exc:
            raise Exception("FT_GetComPortNumber is only available on windows") from exc
        return m.value

    def eeProgram(self, progdata: _ft.ft_program_data | None = None, **kwds) -> None:
        """Program the EEPROM with custom data. If SerialNumber is null, a new
        serial number is generated from ManufacturerId"""
        if progdata is None:
            progdata = ft_program_data(**kwds)
        #        if self.devInfo['type'] == 4:
        #            version = 1
        #        elif self.devInfo['type'] == 5:
        #            version = 2
        #        else:
        #            version = 0
        progdata.Signature1 = _ft.DWORD(0)
        progdata.Signature2 = _ft.DWORD(0xFFFFFFFF)
        progdata.Version = _ft.DWORD(2)
        call_ft(_ft.FT_EE_Program, self.handle, progdata)

    def eeRead(self) -> _ft.ft_program_data:
        """Get the program information from the EEPROM"""
        #        if self.devInfo['type'] == 4:
        #            version = 1
        #        elif self.devInfo['type'] == 5:
        #            version = 2
        #        else:
        #            version = 0

        Manufacturer = c.create_string_buffer(defines.MAX_DESCRIPTION_SIZE)
        ManufacturerId = c.create_string_buffer(defines.MAX_DESCRIPTION_SIZE)
        Description = c.create_string_buffer(defines.MAX_DESCRIPTION_SIZE)
        SerialNumber = c.create_string_buffer(defines.MAX_DESCRIPTION_SIZE)
        progdata = ft_program_data(
            **ProgramData(
                Signature1=0,
                Signature2=0xFFFFFFFF,
                Version=4,
                Manufacturer=c.cast(Manufacturer, c.c_char_p),
                ManufacturerId=c.cast(ManufacturerId, c.c_char_p),
                Description=c.cast(Description, c.c_char_p),
                SerialNumber=c.cast(SerialNumber, c.c_char_p),
            )
        )

        call_ft(_ft.FT_EE_Read, self.handle, c.byref(progdata))
        return progdata

    def eeUASize(self) -> int:
        """Get the EEPROM user area size"""
        uasize = _ft.DWORD()
        call_ft(_ft.FT_EE_UASize, self.handle, c.byref(uasize))
        return uasize.value

    def eeUAWrite(self, data: bytes) -> None:
        """Write data to the EEPROM user area. data must be a bytes object with
        appropriate byte values"""
        buf = (c.c_ubyte * len(data)).from_buffer_copy(data)
        call_ft(_ft.FT_EE_UAWrite, self.handle, buf, len(data))

    def eeUARead(self, b_to_read: int) -> bytes:
        """Read b_to_read bytes from the EEPROM user area"""
        b_read = _ft.DWORD()
        # buf = c.create_string_buffer(b_to_read)
        buf = (c.c_ubyte * (b_to_read + 1))()
        call_ft(
            _ft.FT_EE_UARead,
            self.handle,
            buf,
            b_to_read,
            c.byref(b_read),
        )
        return bytes(buf[: b_read.value])

    def __exit__(
        self,
        __exc_type: type[BaseException] | None,
        __exc_value: BaseException | None,
        __traceback: TracebackType | None,
    ) -> bool | None:
        """Close the device when exiting the context manager"""
        self.close()
        return super().__exit__(__exc_type, __exc_value, __traceback)


__all__ = [
    "call_ft",
    "listDevices",
    "getLibraryVersion",
    "createDeviceInfoList",
    "getDeviceInfoDetail",
    "open",
    "openEx",
    "FTD2XX",
    "DeviceError",
    "ft_program_data",
]
if sys.platform == "win32":
    __all__ += ["w32CreateFile"]
else:
    __all__ += ["getVIDPID", "setVIDPID"]



from enum import unique, IntFlag, IntEnum
import sys

# Statuses
OK = 0
INVALID_HANDLE = 1
DEVICE_NOT_FOUND = 2
DEVICE_NOT_OPENED = 3
IO_ERROR = 4
INSUFFICIENT_RESOURCES = 5
INVALID_PARAMETER = 6
INVALID_BAUD_RATE = 7
DEVICE_NOT_OPENED_FOR_ERASE = 8
DEVICE_NOT_OPENED_FOR_WRITE = 9
FAILED_TO_WRITE_DEVICE = 10
EEPROM_READ_FAILED = 11
EEPROM_WRITE_FAILED = 12
EEPROM_ERASE_FAILED = 13
EEPROM_NOT_PRESENT = 14
EEPROM_NOT_PROGRAMMED = 15
INVALID_ARGS = 16
NOT_SUPPORTED = 17
OTHER_ERROR = 18

# List Devices flags
LIST_NUMBER_ONLY = 0x80000000
LIST_BY_INDEX = 0x40000000
LIST_ALL = 0x20000000


@unique
class OpenExFlags(IntFlag):
    """Used to indicate the type of identifier being passed to FT_OpenEx."""

    OPEN_BY_SERIAL_NUMBER = 1
    OPEN_BY_DESCRIPTION = 2

    if sys.platform == "win32":
        OPEN_BY_LOCATION = 4


OPEN_BY_SERIAL_NUMBER = OpenExFlags.OPEN_BY_SERIAL_NUMBER
OPEN_BY_DESCRIPTION = OpenExFlags.OPEN_BY_DESCRIPTION
if sys.platform == "win32":
    OPEN_BY_LOCATION = OpenExFlags.OPEN_BY_LOCATION


@unique
class ModemStatus(IntFlag):
    #: Clear to Send
    CTS = 0x10

    #: Data Set Ready
    DSR = 0x20

    #: Ring Indicator
    RI = 0x40

    #: Data Carrier Detect
    DCD = 0x80

    #: Data Ready
    DR = 0x100

    #: Overrun Error
    OE = 0x200

    #: Parity Error
    PE = 0x400

    #: Framing Error
    FE = 0x800

    #: Break Interrupt
    BI = 0x1000

    #: Transmitter Holding Register
    THRE = 0x2000

    #: Transmitter Empty
    TEMT = 0x4000

    #: Receiver FIFO Error
    RCVE = 0x8000


@unique
class Device(IntEnum):
    FT_232BM = 0
    FT_232AM = 1
    FT_100AX = 2
    UNKNOWN = 3
    FT_2232C = 4
    FT_232R = 5
    FT_2232H = 6
    FT_4232H = 7
    FT_232H = 8
    FT_X_SERIES = 9


# Device Identifiers
DEVICE_232BM = Device.FT_232BM
DEVICE_232AM = Device.FT_232AM
DEVICE_100AX = Device.FT_100AX
DEVICE_UNKNOWN = Device.UNKNOWN
DEVICE_2232C = Device.FT_2232C
DEVICE_232R = Device.FT_232R
DEVICE_2232H = Device.FT_2232H
DEVICE_4232H = Device.FT_4232H
DEVICE_232H = Device.FT_232H
DEVICE_X_SERIES = Device.FT_X_SERIES


@unique
class Status(IntEnum):
    OK = 0
    INVALID_HANDLE = 1
    DEVICE_NOT_FOUND = 2
    DEVICE_NOT_OPENED = 3
    IO_ERROR = 4
    INSUFFICIENT_RESOURCES = 5
    INVALID_PARAMETER = 6
    INVALID_BAUD_RATE = 7
    DEVICE_NOT_OPENED_FOR_ERASE = 8
    DEVICE_NOT_OPENED_FOR_WRITE = 9
    FAILED_TO_WRITE_DEVICE = 10
    EEPROM_READ_FAILED = 11
    EEPROM_WRITE_FAILED = 12
    EEPROM_ERASE_FAILED = 13
    EEPROM_NOT_PRESENT = 14
    EEPROM_NOT_PROGRAMMED = 15
    INVALID_ARGS = 16
    NOT_SUPPORTED = 17
    OTHER_ERROR = 18


# Driver Types
DRIVER_TYPE_D2XX = 0
DRIVER_TYPE_VCP = 1

# Word Lengths
BITS_8 = 8
BITS_7 = 7

# Stop Bits
STOP_BITS_1 = 0
STOP_BITS_2 = 2

# Parity
PARITY_NONE = 0
PARITY_ODD = 1
PARITY_EVEN = 2
PARITY_MARK = 3
PARITY_SPACE = 4

# Flow Control
FLOW_NONE = 0x0000
FLOW_RTS_CTS = 0x0100
FLOW_DTR_DSR = 0x0200
FLOW_XON_XOFF = 0x0400

# Purge RX and TX Buffers
PURGE_RX = 1
PURGE_TX = 2

# Notification Events
EVENT_RXCHAR = 1
EVENT_MODEM_STATUS = 2
EVENT_LINE_STATUS = 4

MAX_DESCRIPTION_SIZE = 256
